@@ -17,21 +17,25 @@ INSERT INTO dem.message_inbox (
 ) VALUES (
 	(select pk from dem.staff where db_user = 'any-doc'),
 	(select pk_type from dem.v_inbox_item_type where type = 'memo' and category = 'administrative'),
-	'Release Notes for GNUmed 1.7.9 (database v22.9)',
-	'GNUmed 1.7.9 Release Notes:
+	'Release Notes for GNUmed 1.8.1 (database v22.12)',
+	'GNUmed 1.8.1 Release Notes:
 
-	1.7.9
+	1.8.1
 
-FIX: billing: invoice ID generation [thanks Marc]
-FIX: dist: GNUmed Manual d/l URL
-FIX: template: letter w/ & w/o Dx-s
-FIX: paperwork: letter receiver dlg address selection
-FIX: EMR/tree: exception on loading visual progress notes
+NEW: tool: read_all_rows_of_table
 
-	22.9
+FIX: bills: failure to generate bill PDFs [thanks Marc]
+FIX: bills: failure to edit bill item date [thanks Marc]
+FIX: bills: failure to edit billable [thanks Marc]
+FIX: export area: fails to load when gm-burn.sh not found [thanks Marc]
+FIX: demographics: failure to edit type of address [thanks Marc]
+FIX: forms: failure to archive generated forms [thanks Marc]
+FIX: demographics: faulty display of patient addresses [thanks Marc]
 
-FIX: clin.v_candidate_diagnoses: missing coalesce()
+	22.12
+
+IMPROVED: robustify backup script
 ');
 
 -- --------------------------------------------------------------
-select gm.log_script_insertion('v22-release_notes-fixup.sql', '22.9');
+select gm.log_script_insertion('v22-release_notes-fixup.sql', '22.12');
